==> parse_this-1.0.1.tar.gz: parse_this-1.0.1/test/parse_this_test.py <==
from parse_this import parse_this, create_parser, parse_class
from parse_this.core import (_get_args_and_defaults, NoDefault,
    _get_args_to_parse, _check_types, _get_arg_parser, _prepare_doc, Self,
    Class, ParseThisError)
import unittest


def parse_me(one, two, three=12):
    """Could use some parsing.

    Args:
        one: some stuff shouldn't be written down
        two: I can turn 2 syllables words into 6 syllables words
        three: I don't like the number three

    Returns:
        the first string argument concatenated with itself 'two' times and the
        last parameters multiplied by itself
    """
    return one * two, three * three


def parse_me_no_defaults(one, two):
    """I don't have any default !

    Args:
        one: some stuff shoudln't be written down
        two: I can turn 2 syllables words into 6 syllables words
    """
    return one * two


def parse_me_no_doc(one, two, three):
    return one * two, three * three


def will_you_dare_parse_me(one, two, three):
    """I am a sneaky function.

    Args:
        one: this one is a no brainer
        three: noticed you're missing docstring for two and
          I'm multiline too!
    """
    return one * two, three * three


def check_my_params_delimiter(one, two, three):
    """I am a sneaky function.

    Args:
        one -- this one is a no brainer even with dashes
        three -- noticed you're missing docstring for two and
          I'm multiline too!
    """
    return one * two, three * three


def blank_line_in_wrong_place(one, two):
    """I put the blank line after arguments ...

    Args:
        one: this help #1

        two: this once won't appear sadly
    """
    return one * two


class TestParseThis(unittest.TestCase):

    def test_prepare_doc_blank_in_wrong_place(self):
        (description, help_msg) = _prepare_doc(blank_line_in_wrong_place,
                                               ["one", "two"], ":")
        self.assertEqual(description, "I put the blank line after arguments ...")
        self.assertEqual(help_msg, {"one": "this help #1",
                                    "two": "Help message for two"})

    def test_get_args_and_default(self):
        args_and_defaults = _get_args_and_defaults(["first", "second", "third"],
                                                   ("default_value",
                                                    "other_default"))
        self.assertListEqual(args_and_defaults, [("first", NoDefault),
                                                 ("second", "default_value"),
                                                 ("third", "other_default")])
        self.assertListEqual(_get_args_and_defaults([], ()), [])

    def test_get_args_to_parse(self):
        self.assertListEqual(_get_args_to_parse(None, []), [])
        self.assertListEqual(
            _get_args_to_parse(None, ["prog", "arg"]), ["arg"])
        self.assertListEqual(_get_args_to_parse(None, ["prog", "arg",
                                                       "--kwargs=12"]),
                              ["arg", "--kwargs=12"])
        self.assertListEqual(_get_args_to_parse([], []), [])
        self.assertListEqual(_get_args_to_parse(["prog", "arg",
                                                 "--kwargs=12"], []),
                              ["prog", "arg", "--kwargs=12"])

    def test_prepare_doc(self):
        (description, help_msg) = _prepare_doc(parse_me,
                                               ["one", "two", "three"], ":")
        self.assertEqual(description, "Could use some parsing.")
        self.assertEqual(help_msg, {"one":"some stuff shouldn't be written down",
                                    "two":"I can turn 2 syllables words into 6 syllables words",
                                    "three": "I don't like the number three"})

    def test_prepare_doc_no_docstring(self):
        (description, help_msg) = _prepare_doc(parse_me_no_doc,
                                               ["one", "two", "three"], ":")
        self.assertEqual(description, "Argument parsing for parse_me_no_doc")
        self.assertEqual(help_msg, {"one": "Help message for one",
                                    "two":  "Help message for two",
                                    "three":  "Help message for three"})

    def test_prepare_doc_will_you_dare(self):
        (description, help_msg) = _prepare_doc(will_you_dare_parse_me,
                                               ["one", "two", "three"], ":")
        self.assertEqual(description, "I am a sneaky function.")
        self.assertEqual(help_msg, {"one": "this one is a no brainer",
                                    "two": "Help message for two",
                                    "three": "noticed you're missing docstring for two and I'm multiline too!"})

    def test_prepare_doc_params_delim(self):
        (description, help_msg) = _prepare_doc(check_my_params_delimiter,
                                               ["one", "two", "three"], "--")
        self.assertEqual(description, "I am a sneaky function.")
        self.assertEqual(help_msg, {"one": "this one is a no brainer even with dashes",
                                    "two": "Help message for two",
                                    "three": "noticed you're missing docstring for two and I'm multiline too!"})

    def test_check_types(self):
        self.assertRaises(ParseThisError, _check_types, [], ["arg_one"], ())
        self.assertRaises(ParseThisError, _check_types, [int, float],
                          ["arg_one"], ())
        try:
            _check_types([], [], ())
            _check_types([int, int], ["arg_one", "arg_two"], ())
            _check_types([int, int], ["arg_one", "arg_two"], (12,))
            _check_types([int], ["arg_one", "arg_two"], (12,))
            _check_types([Self, int, int], ["self", "arg_one", "arg_two"], ())
        except Exception as exception:
            self.fail("_check_types should not have raised: %s" % exception)

    def test_parsing_no_defaults(self):
        parser = _get_arg_parser(parse_me, [str, int], [("one", NoDefault),
                                                        ("two", NoDefault)],
                                 ":")
        namespace = parser.parse_args("yes 2".split())
        self.assertEqual(namespace.one, "yes")
        self.assertEqual(namespace.two, 2)

    def test_namespace_no_option(self):
        parser = _get_arg_parser(parse_me, [str, int], [("one", NoDefault),
                                                        ("two", NoDefault),
                                                        ("three", 12)], ":")
        namespace = parser.parse_args("yes 42".split())
        self.assertEqual(namespace.one, "yes")
        self.assertEqual(namespace.two, 42)
        self.assertEqual(namespace.three, 12)

    def test_namespace(self):
        parser = _get_arg_parser(parse_me, [str, int], [("one", NoDefault),
                                                        ("two", NoDefault),
                                                        ("three", 12)], ":")
        namespace = parser.parse_args("no 12 --three=23".split())
        self.assertEqual(namespace.one, "no")
        self.assertEqual(namespace.two, 12)
        self.assertEqual(namespace.three, 23)

    def test_return_value(self):
        self.assertEqual(parse_this(parse_me, [str, int], "yes 2".split()),
                          ("yesyes", 144))
        self.assertEqual(parse_this(parse_me, [str, int],
                                     "no 3 --three 2".split()),
                          ("nonono", 4))


@create_parser(str, int)
def iam_parseable(one, two, three=12):
    """I too want to be parseable.

    Args:
      one: the one and only
      two: for the money
      three: don't like the number three
    """
    return one * two, three * three


@parse_class(description="Hello World", parse_private=True)
class NeedParsing(object):
    """This will be used as the parser description."""

    @create_parser(Self, int)
    def __init__(self, four):
        """
        Args:
            four: an int that will be used to multiply stuff
        """
        self._four = four

    @create_parser(Self, int)
    def multiply_self_arg(self, num):
        return self._four * num

    @create_parser(Self, int)
    def _private_method(self, num):
        return self._four * num

    @create_parser(Self)
    def __str__(self):
        return str(self._four)

    @create_parser(Self, str, int)
    def could_you_parse_me(self, one, two, three=12):
        """I would like some arg parsing please.

        Args:
          one: and only one
          two: will never be first
          three: I don't like the number three
        """
        return one * two, three * three

    @classmethod
    @create_parser(Class, str, int)
    def parse_me_if_you_can(cls, one, two, three=12):
        return one * two, three * three


@parse_class()
class ShowMyDocstring(object):
    """This should be the parser description"""

    @create_parser(Self, int)
    def _will_not_appear(self, num):
        return num * num

    @create_parser(Self)
    def __str__(self):
        return self.__class__.__name__


@parse_class()
class NeedInitDecorator(object):

    def __init__(self, val):
        self._val = val

    @create_parser(Self, int)
    def do_stuff(self, num, div=2):
        return self._val * num / div


class TestParseable(unittest.TestCase):

    def test_class_decorator_description(self):
        self.assertEqual(NeedParsing.parser.description, "Hello World")
        self.assertEqual(ShowMyDocstring.parser.description,
                         "This should be the parser description")

    def test_class_is_decorated(self):
        self.assertTrue(hasattr(NeedParsing, "parser"))
        self.assertTrue(hasattr(NeedParsing(12), "parser"))
        self.assertTrue(hasattr(ShowMyDocstring, "parser"))

    def test_subparsers(self):
        parser = NeedParsing.parser
        self.assertEqual(parser.call("12 multiply-self-arg 2".split()), 24)
        self.assertEqual(parser.call("12 could-you-parse-me yes 2 --three 4".split()),
                         ("yesyes", 16))

    def test_private_method_are_exposed(self):
        parser = NeedParsing.parser
        self.assertEqual(parser.call("12 private-method 2".split()), 24)

    def test_special_method_is_exposed(self):
        parser = NeedParsing.parser
        self.assertEqual(parser.call("12 str".split()), "12")

    def test_private_method_are_not_exposed(self):
        with self.assertRaises(SystemExit):
            ShowMyDocstring.parser.parse_args("will-not-appear 12".split())
        with self.assertRaises(SystemExit):
            ShowMyDocstring.parser.parse_args("str".split())

    def test_parseable(self):
        parser = iam_parseable.parser
        namespace = parser.parse_args("yes 2 --three 3".split())
        self.assertEqual(namespace.one, "yes")
        self.assertEqual(namespace.two, 2)
        self.assertEqual(namespace.three, 3)
        self.assertEqual(iam_parseable("yes", 2, 3), ("yesyes", 9))

    def test_parseable_method(self):
        need_parsing = NeedParsing(12)
        parser = need_parsing.could_you_parse_me.parser
        namespace = parser.parse_args("yes 2 --three 3".split())
        self.assertEqual(namespace.one, "yes")
        self.assertEqual(namespace.two, 2)
        self.assertEqual(namespace.three, 3)
        self.assertEqual(need_parsing.could_you_parse_me(2, "yes", 3),
                         ("yesyes", 9))

    def test_parseable_class(self):
        parser = NeedParsing.parse_me_if_you_can.parser
        namespace = parser.parse_args("yes 2 --three 3".split())
        self.assertEqual(namespace.one, "yes")
        self.assertEqual(namespace.two, 2)
        self.assertEqual(namespace.three, 3)
        self.assertEqual(NeedParsing.parse_me_if_you_can(2, "yes", 3),
                         ("yesyes", 9))

    def test_init_need_decoration(self):
        with self.assertRaises(ParseThisError):
            NeedInitDecorator.parser.call("do-stuff 12".split())

    def test_need_init_decorator_with_instance(self):
        instance = NeedInitDecorator(2)
        self.assertEqual(NeedInitDecorator.parser.call("do-stuff 12".split(),
                                                       instance), 12)
        self.assertEqual(NeedInitDecorator.parser.call("do-stuff 12 --div 3".split(),
                                                       instance), 8)

if __name__ == "__main__":
    unittest.main()

==> parse_this-1.0.2.tar.gz: parse_this-1.0.2/test/parse_this_test.py <==
from parse_this import parse_this, create_parser, parse_class
from parse_this.core import Self, Class, ParseThisError
from test.core_test import captured_output
import unittest


def parse_me(one, two, three=12):
    """Could use some parsing.

    Args:
        one: some stuff shouldn't be written down
        two: I can turn 2 syllables words into 6 syllables words
        three: I don't like the number three

    Returns:
        the first string argument concatenated with itself 'two' times and the
        last parameters multiplied by itself
    """
    return one * two, three * three


class TestParseThis(unittest.TestCase):

    def test_parse_this_return_value(self):
        self.assertEqual(parse_this(parse_me, [str, int], "yes 2".split()),
                          ("yesyes", 144))
        self.assertEqual(parse_this(parse_me, [str, int],
                                     "no 3 --three 2".split()),
                          ("nonono", 4))


@create_parser(str, int)
def i_am_parseable(one, two, three=12):
    """I too want to be parseable.

    Args:
      one: the one and only
      two: for the money
      three: don't like the number three
    """
    return one * two, three * three


class Dummy(object):

    def __init__(self, a):
        self._a = a

    @create_parser(Self, int, params_delim="--")
    def multiply_all(self, b, c=2):
        """Will multiply everything!

        Args:
            b -- the Queen B
            c -- a useless value

        Returns:
            Everything multiplied
        """
        return self._a * b * c

    @classmethod
    @create_parser(Class, int)
    def mult(cls, d, e=2):
        return d * e


class NeedParseClassDecorator(object):

    @create_parser(Self, int)
    def __init__(self, a):
        self._a = a


class TestCreateParser(unittest.TestCase):

    def test_create_parser_on_function(self):
        parser = i_am_parseable.parser
        self.assertEqual(parser.description, "I too want to be parseable.")
        namespace = parser.parse_args("yes 2 --three 3".split())
        self.assertEqual(i_am_parseable(namespace.one, namespace.two,
                                        namespace.three), ("yesyes", 9))

    def test_create_parser_on_method(self):
        parser = Dummy.multiply_all.parser
        self.assertEqual(parser.description, "Will multiply everything!")
        self.assertEqual(parser.call(Dummy(12), ["2"]), 48)

    def test_create_parser_on_classmethod(self):
        parser = Dummy.mult.parser
        self.assertEqual(parser.call(Dummy, "2 --e 2".split()), 4)

    def test_create_parser_on_init(self):
        parser = NeedParseClassDecorator.__init__.parser
        self.assertRaises(ParseThisError, parser.call, None, ["2"])


@parse_class(description="Hello World", parse_private=True)
class NeedParsing(object):
    """This will be used as the parser description."""

    @create_parser(Self, int)
    def __init__(self, four):
        """
        Args:
            four: an int that will be used to multiply stuff
        """
        self._four = four

    @create_parser(Self, int)
    def multiply_self_arg(self, num):
        return self._four * num

    @create_parser(Self, int)
    def _private_method(self, num):
        return self._four * num

    @create_parser(Self)
    def __str__(self):
        return str(self._four)

    @create_parser(Self, str, int)
    def could_you_parse_me(self, one, two, three=12):
        """I would like some arg parsing please.

        Args:
          one: and only one
          two: will never be first
          three: I don't like the number three
        """
        return one * two, three * three

    @classmethod
    @create_parser(Class, str, int)
    def parse_me_if_you_can(cls, one, two, three=12):
        return one * two, three * three


@parse_class()
class ShowMyDocstring(object):
    """This should be the parser description"""

    @create_parser(Self, int)
    def _will_not_appear(self, num):
        return num * num

    @create_parser(Self)
    def __str__(self):
        return self.__class__.__name__


@parse_class()
class NeedInitDecorator(object):

    def __init__(self, val):
        self._val = val

    @create_parser(Self, int)
    def do_stuff(self, num, div=2):
        return self._val * num / div


class TestParseClass(unittest.TestCase):

    def test_parse_class_description(self):
        self.assertEqual(NeedParsing.parser.description, "Hello World")
        self.assertEqual(ShowMyDocstring.parser.description,
                         "This should be the parser description")

    def test_parse_class_add_parser(self):
        self.assertTrue(hasattr(NeedParsing, "parser"))
        self.assertTrue(hasattr(NeedParsing(12), "parser"))
        self.assertTrue(hasattr(ShowMyDocstring, "parser"))

    def test_parse_class_subparsers(self):
        parser = NeedParsing.parser
        self.assertEqual(parser.call("12 multiply-self-arg 2".split()), 24)
        self.assertEqual(parser.call("12 could-you-parse-me yes 2 --three 4".split()),
                         ("yesyes", 16))

    def test_parse_class_expose_private_method(self):
        parser = NeedParsing.parser
        self.assertEqual(parser.call("12 private-method 2".split()), 24)

    def test_parse_class_expose_special_method(self):
        parser = NeedParsing.parser
        self.assertEqual(parser.call("12 str".split()), "12")

    def test_parse_class_do_not_expose_private_methods(self):
        with captured_output():
            with self.assertRaises(SystemExit):
                ShowMyDocstring.parser.parse_args("will-not-appear 12".split())
            with self.assertRaises(SystemExit):
                ShowMyDocstring.parser.parse_args("str".split())

    def test_parse_class_method_is_still_parseable(self):
        need_parsing = NeedParsing(12)
        parser = need_parsing.could_you_parse_me.parser
        namespace = parser.parse_args("yes 2 --three 3".split())
        self.assertEqual(namespace.one, "yes")
        self.assertEqual(namespace.two, 2)
        self.assertEqual(namespace.three, 3)
        self.assertEqual(need_parsing.could_you_parse_me("yes", 2, 3),
                         ("yesyes", 9))

    def test_parse_class_init_need_decoration(self):
        with self.assertRaises(ParseThisError):
            NeedInitDecorator.parser.call("do-stuff 12".split())

    def test_parse_class_need_init_decorator_with_instance(self):
        instance = NeedInitDecorator(2)
        self.assertEqual(NeedInitDecorator.parser.call("do-stuff 12".split(),
                                                       instance), 12)
        self.assertEqual(NeedInitDecorator.parser.call("do-stuff 12 --div 3".split(),
                                                       instance), 8)

    def test_parse_class_classmethod_are_not_sub_command(self):
        with captured_output():
            with self.assertRaises(SystemExit):
                NeedParsing.parser.call("12 parse-me-if-you-can one 2")

if __name__ == "__main__":
    unittest.main()
